@@ -1,21 +1,19 @@
 from functools import wraps
-from typing import List, Sequence, Union
+from typing import Dict, List, Sequence, Union
 
 
 class JsonOperationError(Exception):
     pass
 
 
-
-
-def get_json_schema():
+def get_json_schema() -> Dict:
     return {
         "$schema": "http://json-schema.org/draft-07/schema",
         "definitions": {
             "operationOrLiteral": {
                 "anyOf": [
                     {"$def": "#"},
-                    {"type": ["string", "number", "boolean", "null"]},
+                    {"type": ["string", "number", "boolean", "null", "array"]},
                 ]
             },
         },
@@ -33,6 +31,7 @@ def get_json_schema():
                         "$def": "#/definitions/operationOrLiteral",
                     },
                 ],
+                "additionalItems": False,
             },
             {
                 "type": "array",
@@ -44,15 +43,15 @@ def get_json_schema():
                         "$def": "#/definitions/operationOrLiteral",
                     },
                 ],
+                "additionalItems": False,
             },
             {
                 "type": "array",
-                "minItems": 3,
+                "minItems": 2,
                 "items": [
                     {"enum": ["and", "or"]},
                     {
                         "$def": "#/definitions/operationOrLiteral",
-
                     },
                     {
                         "$def": "#/definitions/operationOrLiteral",
@@ -62,11 +61,15 @@ def get_json_schema():
         ],
     }
 
+
 def _get_type_from_val(val):
     if _is_number(val):
         return "number"
     elif isinstance(val, str):
         return "string"
+    elif isinstance(val, bool):
+        return "boolean"
+
 
 def _use_type_from_val(operator):
     return operator in {">", "<", ">=", "<=", "=", "==", "!="}
@@ -76,13 +79,14 @@ def _get_type_from_operator(operator, index):
     if operator in {">", "<", ">=", "<="}:
         return "number"
     elif operator in {"=", "==", "!="}:
-        return ["number", "string"]
+        return ["number", "string", "boolean"]
     elif operator in {"in"}:
         if index == 0:
             return ["string", "number"]
         elif index == 1:
             return ["string", "array"]
 
+
 def _and(*args):
     return all(args)
 
@@ -180,6 +184,8 @@ _operators = {
     "in": _in,
 }
 
+_nesting_operators = {"and", "or"}
+
 
 def _get_key(context, key, default=None):
     # Gets the key from the context dictionary
@@ -195,7 +201,7 @@ def _get_key(context, key, default=None):
         return context
 
 
-def get_keys(json_operation):
+def get_keys(json_operation: List) -> List[Dict]:
     operator, *unparsed = json_operation
 
     keys = []
@@ -213,7 +219,14 @@ def get_keys(json_operation):
                     result["index"] = index
             subkeys += results
         else:
-            type_from_val = _get_type_from_val(item)
+            new_type_from_val = _get_type_from_val(item)
+            if type_from_val and new_type_from_val != type_from_val:
+                raise JsonOperationError(
+                    f'Operation {json_operation}" has values of different types. `{val}` is of type "{type_from_val}". '
+                    f'`{item}` is of type "{new_type_from_val}"'
+                )
+            type_from_val = new_type_from_val
+            val = item
 
     for key in subkeys:
         if key["type"] is None:
@@ -225,7 +238,11 @@ def get_keys(json_operation):
 
             # Check to make sure the types are compatible
             if (
-                    type_from_operator and type_from_val and type_from_operator != type_from_val and type_from_val not in type_from_operator):
+                type_from_operator
+                and type_from_val
+                and type_from_operator != type_from_val
+                and type_from_val not in type_from_operator
+            ):
                 raise JsonOperationError(
                     f'Operation {json_operation} expects type "{type_from_operator}". But `{val}` is of type "{type_from_val}"'
                 )
@@ -233,13 +250,22 @@ def get_keys(json_operation):
     return keys + subkeys
 
 
-def execute(json_operation, context):
+def execute(json_operation: List, context) -> bool:
     # Stop the recursion, we have reached a literal
     if not isinstance(json_operation, list):
         return json_operation
 
     operator, *unparsed = json_operation
-    values = [execute(val, context) for val in unparsed]
+    if operator in _nesting_operators:
+        values = [execute(val, context) for val in unparsed]
+    else:
+        values = []
+        for val in unparsed:
+            # Check if it's a key operation
+            if isinstance(val, list) and 2 >= len(val) <= 3 and val[0] == "key":
+                values.append(_get_key(context, *val[1:]))
+            else:
+                values.append(val)
 
     if operator == "key":
         return _get_key(context, *values)
